@@ -37,6 +37,8 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import java.util.concurrent.atomic.AtomicLong;
+
+import lombok.Getter;
 import org.apache.bookkeeper.bookie.GarbageCollector.GarbageCleaner;
 import org.apache.bookkeeper.bookie.stats.GarbageCollectorStats;
 import org.apache.bookkeeper.conf.ServerConfiguration;
@@ -65,16 +67,21 @@ public class GarbageCollectorThread extends SafeRunnable {
     final long gcWaitTime;
 
     // Compaction parameters
+    boolean isForceMinorCompactionAllow = false;
     boolean enableMinorCompaction = false;
     final double minorCompactionThreshold;
     final long minorCompactionInterval;
+    final long minorCompactionMaxTimeMillis;
     long lastMinorCompactionTime;
 
+    boolean isForceMajorCompactionAllow = false;
     boolean enableMajorCompaction = false;
     final double majorCompactionThreshold;
     final long majorCompactionInterval;
+    long majorCompactionMaxTimeMillis;
     long lastMajorCompactionTime;
 
+    @Getter
     final boolean isForceGCAllowWhenNoSpace;
 
     // Entry Logger Handle
@@ -175,6 +182,10 @@ public class GarbageCollectorThread extends SafeRunnable {
         majorCompactionThreshold = conf.getMajorCompactionThreshold();
         majorCompactionInterval = conf.getMajorCompactionInterval() * SECOND;
         isForceGCAllowWhenNoSpace = conf.getIsForceGCAllowWhenNoSpace();
+        majorCompactionMaxTimeMillis = conf.getMajorCompactionMaxTimeMillis();
+        minorCompactionMaxTimeMillis = conf.getMinorCompactionMaxTimeMillis();
+
+        boolean isForceAllowCompaction = conf.isForceAllowCompaction();
 
         AbstractLogCompactor.LogRemovalListener remover = new AbstractLogCompactor.LogRemovalListener() {
             @Override
@@ -200,6 +211,15 @@ public class GarbageCollectorThread extends SafeRunnable {
             enableMinorCompaction = true;
         }
 
+        if (isForceAllowCompaction) {
+            if (minorCompactionThreshold > 0 && minorCompactionThreshold < 1.0f) {
+                isForceMinorCompactionAllow = true;
+            }
+            if (majorCompactionThreshold > 0 && majorCompactionThreshold < 1.0f) {
+                isForceMajorCompactionAllow = true;
+            }
+        }
+
         if (majorCompactionInterval > 0 && majorCompactionThreshold > 0) {
             if (majorCompactionThreshold > 1.0f) {
                 throw new IOException("Invalid major compaction threshold "
@@ -341,23 +361,25 @@ public class GarbageCollectorThread extends SafeRunnable {
         }
 
         long curTime = System.currentTimeMillis();
-        if (enableMajorCompaction && (!suspendMajor)
-            && (force || curTime - lastMajorCompactionTime > majorCompactionInterval)) {
+        if (((isForceMajorCompactionAllow && force)
+                || (enableMajorCompaction && (force || curTime - lastMajorCompactionTime > majorCompactionInterval)))
+                && (!suspendMajor)) {
             // enter major compaction
             LOG.info("Enter major compaction, suspendMajor {}", suspendMajor);
             majorCompacting.set(true);
-            doCompactEntryLogs(majorCompactionThreshold);
+            doCompactEntryLogs(majorCompactionThreshold, majorCompactionMaxTimeMillis);
             lastMajorCompactionTime = System.currentTimeMillis();
             // and also move minor compaction time
             lastMinorCompactionTime = lastMajorCompactionTime;
             gcStats.getMajorCompactionCounter().inc();
             majorCompacting.set(false);
-        } else if (enableMinorCompaction && (!suspendMinor)
-            && (force || curTime - lastMinorCompactionTime > minorCompactionInterval)) {
+        } else if (((isForceMinorCompactionAllow && force)
+                || (enableMinorCompaction && (force || curTime - lastMinorCompactionTime > minorCompactionInterval)))
+                && (!suspendMinor)) {
             // enter minor compaction
             LOG.info("Enter minor compaction, suspendMinor {}", suspendMinor);
             minorCompacting.set(true);
-            doCompactEntryLogs(minorCompactionThreshold);
+            doCompactEntryLogs(minorCompactionThreshold, minorCompactionMaxTimeMillis);
             lastMinorCompactionTime = System.currentTimeMillis();
             gcStats.getMinorCompactionCounter().inc();
             minorCompacting.set(false);
@@ -427,7 +449,7 @@ public class GarbageCollectorThread extends SafeRunnable {
      * </p>
      */
     @VisibleForTesting
-    void doCompactEntryLogs(double threshold) {
+    void doCompactEntryLogs(double threshold, long maxTimeMillis) {
         LOG.info("Do compaction to compact those files lower than {}", threshold);
 
         // sort the ledger meta by usage in ascending order.
@@ -437,31 +459,59 @@ public class GarbageCollectorThread extends SafeRunnable {
 
         final int numBuckets = 10;
         int[] entryLogUsageBuckets = new int[numBuckets];
+        int[] compactedBuckets = new int[numBuckets];
+
+        long start = System.currentTimeMillis();
+        long end = start;
+        long timeDiff = 0;
 
         for (EntryLogMetadata meta : logsToCompact) {
-            int bucketIndex = Math.min(
-                    numBuckets - 1,
-                    (int) Math.ceil(meta.getUsage() * numBuckets));
+            int bucketIndex = calculateUsageIndex(numBuckets, meta.getUsage());
             entryLogUsageBuckets[bucketIndex]++;
 
-            if (meta.getUsage() >= threshold) {
+            if (timeDiff < maxTimeMillis) {
+                end = System.currentTimeMillis();
+                timeDiff = end - start;
+            }
+            if (meta.getUsage() >= threshold || (maxTimeMillis > 0 && timeDiff > maxTimeMillis) || !running) {
+                // We allow the usage limit calculation to continue so that we get a accurate
+                // report of where the usage was prior to running compaction.
                 continue;
             }
             if (LOG.isDebugEnabled()) {
-                LOG.debug("Compacting entry log {} below threshold {}", meta.getEntryLogId(), threshold);
+                LOG.debug("Compacting entry log {} with usage {} below threshold {}",
+                        meta.getEntryLogId(), meta.getUsage(), threshold);
             }
 
             long priorRemainingSize = meta.getRemainingSize();
             compactEntryLog(meta);
             gcStats.getReclaimedSpaceViaCompaction().add(meta.getTotalSize() - priorRemainingSize);
-
-            if (!running) { // if gc thread is not running, stop compaction
-                return;
+            compactedBuckets[bucketIndex]++;
+        }
+        if (LOG.isDebugEnabled()) {
+            if (!running) {
+                LOG.debug("Compaction exited due to gc not running");
+            }
+            if (timeDiff > maxTimeMillis) {
+                LOG.debug("Compaction ran for {}ms but was limited by {}ms", timeDiff, maxTimeMillis);
             }
         }
         LOG.info(
-                "Compaction: entry log usage buckets[10% 20% 30% 40% 50% 60% 70% 80% 90% 100%] = {}",
-                entryLogUsageBuckets);
+                "Compaction: entry log usage buckets[10% 20% 30% 40% 50% 60% 70% 80% 90% 100%] = {}, compacted {}",
+                entryLogUsageBuckets, compactedBuckets);
+    }
+
+    /**
+     * Calculate the index for the batch based on the usage between 0 and 1.
+     *
+     * @param numBuckets Number of reporting buckets.
+     * @param usage 0.0 - 1.0 value representing the usage of the entry log.
+     * @return index based on the number of buckets The last bucket will have the 1.0 if added.
+     */
+    int calculateUsageIndex(int numBuckets, double usage) {
+        return Math.min(
+                numBuckets - 1,
+                (int) Math.floor(usage * numBuckets));
     }
 
     /**
