@@ -52,6 +52,7 @@ import org.apache.bookkeeper.bookie.LedgerDirsManager.NoWritableLedgerDirExcepti
 import org.apache.bookkeeper.bookie.stats.JournalStats;
 import org.apache.bookkeeper.common.collections.BlockingMpscQueue;
 import org.apache.bookkeeper.common.collections.RecyclableArrayList;
+import org.apache.bookkeeper.common.util.MemoryLimitController;
 import org.apache.bookkeeper.common.util.affinity.CpuAffinity;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.WriteCallback;
@@ -295,7 +296,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     /**
      * Journal Entry to Record.
      */
-    private static class QueueEntry implements Runnable {
+    static class QueueEntry implements Runnable {
         ByteBuf entry;
         long ledgerId;
         long entryId;
@@ -390,7 +391,6 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                     if (qe != null) {
                         cbThreadPool.execute(qe);
                     }
-                    journalStats.getJournalCbQueueSize().inc();
                 }
 
                 return forceWriteWaiters.size();
@@ -632,6 +632,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     volatile boolean running = true;
     private final LedgerDirsManager ledgerDirsManager;
     private final ByteBufAllocator allocator;
+    private final MemoryLimitController memoryLimitController;
 
     // Expose Stats
     private final JournalStats journalStats;
@@ -656,6 +657,9 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             forceWriteRequests = new ArrayBlockingQueue<>(conf.getJournalQueueSize());
         }
 
+        // Adjust the journal max memory in case there are multiple journals configured.
+        long journalMaxMemory = conf.getJournalMaxMemorySizeMb() / conf.getJournalDirNames().length * 1024 * 1024;
+        this.memoryLimitController = new MemoryLimitController(journalMaxMemory);
         this.ledgerDirsManager = ledgerDirsManager;
         this.conf = conf;
         this.journalDirectory = journalDirectory;
@@ -861,23 +865,29 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     public void logAddEntry(long ledgerId, long entryId, ByteBuf entry,
                             boolean ackBeforeSync, WriteCallback cb, Object ctx)
             throws InterruptedException {
-        //Retain entry until it gets written to journal
+        // Retain entry until it gets written to journal
         entry.retain();
 
         journalStats.getJournalQueueSize().inc();
+        journalStats.getJournalCbQueueSize().inc();
+
+        memoryLimitController.reserveMemory(entry.readableBytes());
+
         queue.put(QueueEntry.create(
                 entry, ackBeforeSync,  ledgerId, entryId, cb, ctx, MathUtils.nowInNano(),
                 journalStats.getJournalAddEntryStats(),
-                journalStats.getJournalQueueSize()));
+                journalStats.getJournalCbQueueSize()));
     }
 
     void forceLedger(long ledgerId, WriteCallback cb, Object ctx) {
-        journalStats.getJournalQueueSize().inc();
         queue.add(QueueEntry.create(
-                null, false /* ackBeforeSync */,  ledgerId,
+                null, false /* ackBeforeSync */, ledgerId,
                 Bookie.METAENTRY_ID_FORCE_LEDGER, cb, ctx, MathUtils.nowInNano(),
                 journalStats.getJournalForceLedgerStats(),
-                journalStats.getJournalQueueSize()));
+                journalStats.getJournalCbQueueSize()));
+        // Increment afterwards because the add operation could fail.
+        journalStats.getJournalQueueSize().inc();
+        journalStats.getJournalCbQueueSize().inc();
     }
 
     /**
@@ -969,6 +979,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                     if (numEntriesToFlush == 0) {
                         qe = queue.take();
                         dequeueStartTime = MathUtils.nowInNano();
+                        journalStats.getJournalQueueSize().dec();
                         journalStats.getJournalQueueStats()
                             .registerSuccessfulEvent(MathUtils.elapsedNanos(qe.enqueueTime), TimeUnit.NANOSECONDS);
                     } else {
@@ -981,6 +992,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                         dequeueStartTime = MathUtils.nowInNano();
 
                         if (qe != null) {
+                            journalStats.getJournalQueueSize().dec();
                             journalStats.getJournalQueueStats()
                                 .registerSuccessfulEvent(MathUtils.elapsedNanos(qe.enqueueTime), TimeUnit.NANOSECONDS);
                         }
@@ -1106,6 +1118,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                      * shouldn't write this special entry
                      * (METAENTRY_ID_LEDGER_EXPLICITLAC) to Journal.
                      */
+                    memoryLimitController.releaseMemory(qe.entry.readableBytes());
                     qe.entry.release();
                 } else if (qe.entryId != Bookie.METAENTRY_ID_FORCE_LEDGER) {
                     int entrySize = qe.entry.readableBytes();
@@ -1121,6 +1134,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
                     bc.write(lenBuff);
                     bc.write(qe.entry);
+                    memoryLimitController.releaseMemory(qe.entry.readableBytes());
                     qe.entry.release();
                 }
 
@@ -1198,4 +1212,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         join();
     }
 
+    long getMemoryUsage() {
+        return memoryLimitController.currentUsage();
+    }
 }
