@@ -31,6 +31,7 @@ import io.netty.buffer.ByteBufAllocator;
 import io.netty.buffer.Unpooled;
 import io.netty.util.concurrent.DefaultThreadFactory;
 
+import java.io.File;
 import java.io.IOException;
 import java.util.Collections;
 import java.util.List;
@@ -59,6 +60,7 @@ import org.apache.bookkeeper.bookie.GarbageCollectorThread;
 import org.apache.bookkeeper.bookie.LastAddConfirmedUpdateNotification;
 import org.apache.bookkeeper.bookie.LedgerCache;
 import org.apache.bookkeeper.bookie.LedgerDirsManager;
+import org.apache.bookkeeper.bookie.LedgerDirsManager.LedgerDirsListener;
 import org.apache.bookkeeper.bookie.LedgerEntryPage;
 import org.apache.bookkeeper.bookie.StateManager;
 import org.apache.bookkeeper.bookie.storage.ldb.DbLedgerStorageDataFormats.LedgerData;
@@ -132,6 +134,8 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     private static final long DEFAULT_MAX_THROTTLE_TIME_MILLIS = TimeUnit.SECONDS.toMillis(10);
 
+    private final long maxReadAheadBytesSize;
+
     public SingleDirectoryDbLedgerStorage(ServerConfiguration conf, LedgerManager ledgerManager,
             LedgerDirsManager ledgerDirsManager, LedgerDirsManager indexDirsManager, StateManager stateManager,
             CheckpointSource checkpointSource, Checkpointer checkpointer, StatsLogger statsLogger,
@@ -153,6 +157,9 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         readCacheMaxSize = readCacheSize;
         readAheadCacheBatchSize = conf.getInt(READ_AHEAD_CACHE_BATCH_SIZE, DEFAULT_READ_AHEAD_CACHE_BATCH_SIZE);
 
+        // Do not attempt to perform read-ahead more than half the total size of the cache
+        maxReadAheadBytesSize = readCacheMaxSize / 2;
+
         long maxThrottleTimeMillis = conf.getLong(DbLedgerStorage.MAX_THROTTLE_TIME_MILLIS,
                 DEFAULT_MAX_THROTTLE_TIME_MILLIS);
         maxThrottleTimeNanos = TimeUnit.MILLISECONDS.toNanos(maxThrottleTimeMillis);
@@ -178,6 +185,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             () -> readCache.size(),
             () -> readCache.count()
         );
+        ledgerDirsManager.addLedgerDirsListener(getLedgerDirsListener());
     }
 
     @Override
@@ -466,7 +474,9 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             int count = 0;
             long size = 0;
 
-            while (count < readAheadCacheBatchSize && currentEntryLogId == firstEntryLogId) {
+            while (count < readAheadCacheBatchSize
+                    && size < maxReadAheadBytesSize
+                    && currentEntryLogId == firstEntryLogId) {
                 ByteBuf entry = entryLogger.internalReadEntry(orginalLedgerId, firstEntryId, currentEntryLocation,
                         false /* validateEntry */);
 
@@ -910,4 +920,62 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         throw new UnsupportedOperationException(
                 "getListOfEntriesOfLedger method is currently unsupported for SingleDirectoryDbLedgerStorage");
     }
+
+    private LedgerDirsManager.LedgerDirsListener getLedgerDirsListener() {
+        return new LedgerDirsListener() {
+
+            @Override
+            public void diskAlmostFull(File disk) {
+                if (gcThread.isForceGCAllowWhenNoSpace()) {
+                    gcThread.enableForceGC();
+                } else {
+                    gcThread.suspendMajorGC();
+                }
+            }
+
+            @Override
+            public void diskFull(File disk) {
+                if (gcThread.isForceGCAllowWhenNoSpace()) {
+                    gcThread.enableForceGC();
+                } else {
+                    gcThread.suspendMajorGC();
+                    gcThread.suspendMinorGC();
+                }
+            }
+
+            @Override
+            public void allDisksFull(boolean highPriorityWritesAllowed) {
+                if (gcThread.isForceGCAllowWhenNoSpace()) {
+                    gcThread.enableForceGC();
+                } else {
+                    gcThread.suspendMajorGC();
+                    gcThread.suspendMinorGC();
+                }
+            }
+
+            @Override
+            public void diskWritable(File disk) {
+                // we have enough space now
+                if (gcThread.isForceGCAllowWhenNoSpace()) {
+                    // disable force gc.
+                    gcThread.disableForceGC();
+                } else {
+                    // resume compaction to normal.
+                    gcThread.resumeMajorGC();
+                    gcThread.resumeMinorGC();
+                }
+            }
+
+            @Override
+            public void diskJustWritable(File disk) {
+                if (gcThread.isForceGCAllowWhenNoSpace()) {
+                    // if a disk is just writable, we still need force gc.
+                    gcThread.enableForceGC();
+                } else {
+                    // still under warn threshold, only resume minor compaction.
+                    gcThread.resumeMinorGC();
+                }
+            }
+        };
+    }
 }
