@@ -27,7 +27,6 @@ import static org.junit.Assume.assumeTrue;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.InetSocketAddress;
 import java.security.cert.Certificate;
 import java.security.cert.X509Certificate;
 import java.util.Arrays;
@@ -60,7 +59,6 @@ import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.proto.BookieConnectionPeer;
 import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.proto.ClientConnectionPeer;
-import org.apache.bookkeeper.proto.PerChannelBookieClient;
 import org.apache.bookkeeper.proto.TestPerChannelBookieClient;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.apache.bookkeeper.test.TestStatsProvider;
@@ -884,10 +882,12 @@ public class TestTLS extends BookKeeperClusterTestCase {
         // verify stats
         for (int i = 0; i < numBookies; i++) {
             BookieServer bookie = bs.get(i);
-            InetSocketAddress addr = bookie.getLocalAddress().getSocketAddress();
             StringBuilder nameBuilder = new StringBuilder(BookKeeperClientStats.CHANNEL_SCOPE)
                     .append(".")
-                    .append(PerChannelBookieClient.buildStatsLoggerScopeName(bookie.getBookieId()))
+                    .append("bookie_")
+                    .append(bookie.getBookieId().toString()
+                    .replace('.', '_')
+                    .replace('-', '_'))
                     .append(".");
 
             // check stats on TLS enabled client
@@ -983,10 +983,12 @@ public class TestTLS extends BookKeeperClusterTestCase {
         }
 
         // check failed handshake counter
-        InetSocketAddress addr = bookie.getLocalAddress().getSocketAddress();
         StringBuilder nameBuilder = new StringBuilder(BookKeeperClientStats.CHANNEL_SCOPE)
                 .append(".")
-                .append(PerChannelBookieClient.buildStatsLoggerScopeName(bookie.getBookieId()))
+                .append("bookie_")
+                .append(bookie.getBookieId().toString()
+                        .replace('.', '_')
+                        .replace('-', '_'))
                 .append(".");
 
         assertEquals("TLS handshake failure expected", 1,
